/*
 * Copyright (C) 2010 SHARP Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.co.sharp.android.samples.pedometer;

import jp.co.sharp.android.hardware.Pedometer;
import android.app.PendingIntent;
import android.app.Service;
import android.appwidget.AppWidgetManager;
import android.content.ActivityNotFoundException;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.view.View;
import android.widget.RemoteViews;
import android.widget.Toast;

/**
 * 歩数計Widgetで数値を取得、表示を行うサービスです。
 * 歩数計の起動状態を確認し、起動中あるいは起動したタイミングで
 * Intent通知を開始する。<br />また、ハンドラを用いて一定周期で画面更新判定を行う。<br />
 */
public class SamplePedometerWidgetService extends Service implements Runnable{

    private Pedometer mPedometer;
    private SamplePedometerWidgetReceiver mReceiver;
    private RemoteViews mView;
    private AppWidgetManager mAppWidgetManager;
    private ComponentName mComponentName;
    private Handler mHandler = new Handler();
    private int mCurrentSteps = 0;
    private int mPrevSteps = 0;
    public static final String ACTION_WIDGET_CLICK =
        "jp.co.sharp.android.SamplePedometer.action.WIDGET_CLICK";
    private static final int HANDLETIME = 1000;
    private static final String PEDOMETER_NOT_SUPPORT = "PEDOMETER_NOT_SUPPORT";

    /** サポート情報 */
    private String[] mSupportedParameters;

    /**
     * Bindが起きた際の処理です。今回は特に何もしません。
     * @param Intent intent
     * @return nullを返却します。
     */
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    /**
     * サービスが作成された時の処理です。
     * onCreate()をオーバーライドしています。<br />
     * 歩数計に関するIntentを受け取るために、
     * IntentFilterをサービスに登録します。
     */
    @Override
    public void onCreate() {
        super.onCreate();
        // インテントフィルターを登録する
        IntentFilter filter = new IntentFilter();
        filter.addAction(Intent.ACTION_SHUTDOWN);
        filter.addAction(Pedometer.ACTION_START);
        filter.addAction(Pedometer.ACTION_STOP);
        filter.addAction(Pedometer.ACTION_MEASURE_CLEAR);
        filter.addAction(Pedometer.ACTION_MEASURE_HOURLY);
        filter.addAction(ACTION_WIDGET_CLICK);

        mReceiver = new SamplePedometerWidgetReceiver(this);
        registerReceiver(mReceiver, filter);

        mAppWidgetManager = AppWidgetManager.getInstance(this);
        mComponentName = new ComponentName(this, SamplePedometerWidgetProvider.class);

        try {

        // Pedometerクラスを取得
        mPedometer = Pedometer.createInstance(this);    /* ★Addon用API使用箇所★ */


        // サポート情報をセット
        mSupportedParameters = new String[] {
                Pedometer.STEPS
        };
        } catch (NoClassDefFoundError e) {
            // 非対応機種の場合
            mPedometer = null;

        }

    }

    /**
     * サービスが開始された時の処理です。
     * onStart()をオーバーライドしています。<br />
     * Intentを受け取るためのIntentFilterを登録しています。
     * @param intent intent
     * @param startId startId
     */
    @Override
    public void onStart(Intent intent, int startId) {
        super.onStart(intent, startId);

    	setRemoteViews();
        final String[] measuredParameterskeys = {
                Pedometer.MEASURE_STATUS
        };
        Bundle measuredData;
        if(null != mPedometer) {
            // 対応機種画面の場合

            measuredData = mPedometer.getParameters(measuredParameterskeys);    /* ★Addon用API使用箇所★ */
            int status = Pedometer.DEVICE_STOPED;

            if(null != measuredData) {
                status = measuredData.getInt(Pedometer.MEASURE_STATUS);
            }
            /*
             * 0:停止状態 [DEVICE_STOPED]
             * 1:起動中   [DEVICE_READY]
             * 2:稼動状態 [DEVICE_STARTED]
             */
            if(status == Pedometer.DEVICE_STARTED) {
                // 歩数計が起動状態であった場合
                setWidgetLayout(Pedometer.ACTION_START);
                // 描画更新スレッドを開始する
                mHandler.removeCallbacks(this);
                mHandler.post(this);
            } else {
                setWidgetLayout(Pedometer.ACTION_STOP);
            }

        } else {
            // 非対応機種の場合
            setWidgetLayout(PEDOMETER_NOT_SUPPORT);
        }
    }

    /**
     * Intentを受け取った場合の処理です。<br />
     * 受け取ったIntentのAction属性を判定し、それぞれの処理を行います。
     * @param context context
     * @param receiveIntent 受け取ったIntent
     */
    public void onReceive(Context context, Intent receivedIntent) {
        String getAction = null;
        if(null != receivedIntent) {
            // レシーブインテントに値が入っていた場合
            getAction = receivedIntent.getAction();
            if(null == getAction) {
                // レシーブインテントに含まれるアクションが空の場合も終了する
                return;
            }
        } else {
            // レシーブインテントが空の場合終了する
            return;
        }

        // レシーブインテントのアクション属性から、処理を分ける
        if(Pedometer.ACTION_MEASURE_HOURLY.equals(getAction)) {
            // 1時間ごとにIntent通知を受け取るときの処理
        } else if(Intent.ACTION_SHUTDOWN.equals(getAction)) {
            // シャットダウンされたときに終了
            unregisterReceiver(mReceiver);
        } else if(Pedometer.ACTION_START.equals(getAction)) {
            // 歩数計が起動されたら500msごとのIntent通知を開始する
        	setRemoteViews();
            switchBroadcast(receivedIntent, getAction);
            mHandler.removeCallbacks(this);
            mHandler.post(this);
        } else if(Pedometer.ACTION_MEASURE_CLEAR.equals(getAction)) {
            // 歩数計データがクリアされたときの処理
        } else if(Pedometer.ACTION_STOP.equals(getAction)) {
            // 歩数計が停止されたら500msごとのIntent通知を停止する
        	setRemoteViews();
            switchBroadcast(receivedIntent, getAction);
            mHandler.removeCallbacks(this);
        } else if(ACTION_WIDGET_CLICK.equals(getAction)) {
            // このWidgetがクリックされた時
            sendPedometerSetting(context);
        }
    }


    /**
     * サービスが終了した時の処理です。
     * onDestroy()をオーバーライドしている。<br />
     * Intentの終了要求を行います。
     * 登録したインテントフィルターを解除します。
     * ハンドラーを終了します。
     */
    @Override
    public void onDestroy() {
        super.onDestroy();
        if(null != mPedometer) {
            // 対応機種画面の場合
            unregisterReceiver(mReceiver);
            mHandler.removeCallbacks(this);
        }
    }

    private void setRemoteViews() {
        mView = new RemoteViews(getPackageName(), R.layout.widgetmain);

        // Widgetがタッチされた際に反応するようなpendingIntentを設置する
        Intent clickIntent = new Intent();
        clickIntent.setAction(ACTION_WIDGET_CLICK);
        PendingIntent pendingIntent = PendingIntent.getBroadcast(this, 0, clickIntent, 0);
        mView.setOnClickPendingIntent(R.id.FrameLayoutSamplePedometerWidget, pendingIntent);
    }
    /**
     * ハンドラーのコールバック処理です。
     * ここで設定した周期（HANDLETIME）で、画面の描画更新判定を行う
     */
    public void run() {
    	// RemoteViewを再取得
    	setRemoteViews();
        // CurrentStepsの更新とmPrevStepsをウィジェットに表示
        updateCount();
        if( mCurrentSteps != mPrevSteps ) {
            // 歩数が変化した場合はWidgetの情報を更新し、mPrevの値も更新する
            updateAppWidget();
            mPrevSteps = mCurrentSteps;
        }
        mHandler.removeCallbacks(this);
        // ここで指定した周期で次のrunのタイミングを決める
        mHandler.postDelayed(this, HANDLETIME);
    }

    /**
     * Widgetの情報を更新
     * @return true : 更新成功
     *         false: 更新失敗
     */
    private boolean updateAppWidget() {
        if(null != mAppWidgetManager && null != mComponentName && null != mView) {
            // Widgetの情報が空でない場合
            mAppWidgetManager.updateAppWidget(
                    mComponentName,
                    mView);
            return true;
        }
        return false;
    }

    /**
     * 実際にカウントしている歩数をmCurrentStepsに格納する。
     * また、mPrevStepsの値をmView上に反映させる。
     */
    private void updateCount() {
        //指定したデータを取得
        Bundle measuredData = mPedometer.getParameters(mSupportedParameters);    /* ★Addon用API使用箇所★ */
        mCurrentSteps = measuredData.getInt(Pedometer.STEPS);
    	mView.setTextViewText(R.id.PedometerWidget_steps, String.valueOf(mCurrentSteps) );
    }

    /**
     * 歩数計ON/OFFでレイアウトを変更する。
     * @param pedometerStatus ACTION_STARTかACITON_STOP
     */
    private void setWidgetLayout(String pedometerStatus) {
        if(Pedometer.ACTION_START.equals(pedometerStatus) ||
        		Pedometer.ACTION_MEASURE_HOURLY.equals(pedometerStatus)) {
            // 歩数計ONのレイアウトに変更する
            mView.setViewVisibility(R.id.LayoutSteps, View.VISIBLE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerOff, View.GONE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerNotSupport, View.GONE);
        } else if(Pedometer.ACTION_STOP.equals(pedometerStatus)) {
            // 歩数計OFFのレイアウトに変更する
            mView.setViewVisibility(R.id.LayoutSteps, View.GONE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerOff, View.VISIBLE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerNotSupport, View.GONE);
        } else {
            // 非対応機種用の画面にする
            mView.setViewVisibility(R.id.LayoutSteps, View.GONE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerOff, View.GONE);
            mView.setViewVisibility(R.id.LinearLayoutPedometerNotSupport, View.VISIBLE);
        }
        updateAppWidget();
    }

    /**
     * 歩数計の起動/終了通知を受け取ったor1時間ごとの更新通知を受け取った場合、画面の表示を切り替える。
     * @param receivedIntent ブロードキャストで受け取ったIntent
     * @param action ブロードキャストで受け取ったIntentのアクション
     */
    private void switchBroadcast(Intent receivedIntent, String action) {
        int status = receivedIntent.getExtras().getInt(Pedometer.STATUS);

        if(status == Pedometer.STATUS_OK) {
            // Setting変更処理が正常終了している場合

            //画面変更
            if(Pedometer.ACTION_START.equals(action)) {
                setWidgetLayout(Pedometer.ACTION_START);
            } else if(Pedometer.ACTION_STOP.equals(action)) {
                setWidgetLayout(Pedometer.ACTION_STOP);
            }

        }
    }

    /**
     * 歩数計設定画面へ遷移させる
     * @param context ブロードキャストを受け取ったcontext
     */
    private void sendPedometerSetting(Context context) {
        String className =
            "jp.co.sharp.android.settings.pedometersetting.PedometerSettingActivity";
        String packageName = "com.android.settings";

        Intent intent = new Intent(Intent.ACTION_MAIN);
        intent.setClassName(packageName, className);
        intent.addCategory(Intent.CATEGORY_LAUNCHER);
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);
        /* 歩数計設定Activityを呼び出す
         * エミュレーターではインスタンスの取得ができても
         * 歩数計設定Activityが居ないため、Activityが存在しない場合は
         * 歩数計設定が起動できないことをToastとして出す
         */
        try {
        context.startActivity(intent);
        } catch (ActivityNotFoundException e) {
            Toast.makeText(context, getString(R.string.canot_pedmeter_setting), Toast.LENGTH_LONG).show();
        }
    }

    /**
     * Widget用ブロードキャストレシーバー
     */
    public class SamplePedometerWidgetReceiver extends BroadcastReceiver {
        private SamplePedometerWidgetService mService;
        /**
         * コンストラクタ
         * @param service
         */
        SamplePedometerWidgetReceiver(SamplePedometerWidgetService service){
            mService = service;
        }

       /**
         * ブロードキャストレシーバー
         * @param context
         * @param intent
         */
        @Override
        public void onReceive(Context context, Intent intent) {
            mService.onReceive(context, intent);
        }
    }
}
